
def import_class(class_path):
    """
    Returns class from the given path.

    For example, in order to get class located at
    ``mynodes.MyNode``:

        try:
            hgrepo = import_class('mynodes.MyNode')
        except ImportError:
            # hadle error
    """
    splitted = class_path.split('.')
    mod_path = '.'.join(splitted[:-1])
    class_name = splitted[-1]
    # import may throw ImportError
    class_mod = __import__(mod_path, {}, {}, [class_name])
    try:
        cls = getattr(class_mod, class_name)
    except AttributeError:
        raise ImportError("Couldn't import %r" % class_path)
    return cls




import datetime
from porunga.utils.backports import get_total_seconds
from porunga.utils.compat import unittest


class TestGetTotalSeconds(unittest.TestCase):

    def assertTotalSecondsEqual(self, timedelta, expected_seconds):
        result = get_total_seconds(timedelta)
        self.assertEqual(result, expected_seconds,
            "We computed %s seconds for %s but expected %s"
            % (result, timedelta, expected_seconds))

    def test_get_total_seconds_returns_proper_value(self):
        self.assertTotalSecondsEqual(datetime.timedelta(seconds=1001), 1001)

    def test_get_total_seconds_returns_proper_value_for_partial_seconds(self):
        self.assertTotalSecondsEqual(datetime.timedelta(seconds=50.65), 50.65)





from __future__ import print_function

import fnmatch
import os
import sys
from os import walk
from datetime import datetime
from glob import glob
from porunga.utils.backports import get_total_seconds
from monolith.cli import arg
from monolith.cli import SingleLabelCommand
from subprocess import Popen, PIPE
from termcolor import colored

joinpath = lambda *p: os.path.join(*p)
abspath = lambda *p: os.path.abspath(joinpath(*p))

LANGUAGES = {
    'python': {
        'fout': '{name}.py',
        'prog': 'python {fout}',
    },
    'ruby': {
        'fout': '{name}.rb',
        'prog': 'ruby {fout}',
    },
    'java': {
        'fin': '{name}.java',
        'fout': '{simplename}',
        'prog': 'java -cp {dirname} {fout}',
        'compiler': 'javac {fin}',
    },
    'objc': {
        'fin': '{name}.m',
        'fout': '{name}.m.out',
        'prog': '{fout}',
        'compiler': 'clang -ObjC -framework Foundation {fin} -o {fout}',
    },
    'c': {
        'fin': '{name}.c',
        'fout': '{name}.c.out',
        'prog': '{fout}',
        'compiler': 'gcc -O2 {fin} -o {fout}',
    },
    'cpp': {
        'fin': '{name}.cpp',
        'fout': '{name}.cpp.out',
        'prog': '{fout}',
        'compiler': 'g++ -O2 {fin} -o {fout}',
    },
}

def curdir():
    return os.path.curdir


class TestCommand(SingleLabelCommand):
    languages = LANGUAGES
    default_language = 'python'

    args = SingleLabelCommand.args + [
        arg('-l', '--lang', type=str, default=default_language,
            help='Available languages: %s (default: %s)' %
                (', '.join((key for key in sorted(languages.keys()))),
                 default_language) ),
        arg('-a', '--all', default=False, action='store_true'),
        arg('-v', '--verbose', default=False, action='store_true'),
        arg('-q', '--quiet', default=False, action='store_true'),
        arg('-c', '--case', type=str),
        arg('-t', '--timeout', type=float, default=3.0),
    ]

    def handle_label(self, label, namespace):
        self.namespace = namespace
        dirname = label or abspath(curdir())

        title = 'Testing %s' % dirname
        subtitle = '=' * len(title)
        self.success_continuation(title)
        self.success_continuation(subtitle)
        print()

        binary = self.get_binary(dirname, namespace.lang)
        self.info("Binary: %s" % binary)
        print()

        time = 0
        total = 0
        fails = 0
        for fin, fout in self.get_test_cases(dirname):
            total += 1
            info = self.test(dirname, binary, fin, fout)
            time += info['time']
            if not info['success']:
                fails += 1

        print()
        self.info("Total time: %.3fs" % time)
        if fails == 0:
            self.success('All %s tests passed' % total)
        else:
            self.error('%s out of %s tests failed' % (fails, total))

    def get_binary(self, dirname, lang):
        dirname = dirname.rstrip('/\\')
        if lang in LANGUAGES:
            data = {'dirname': dirname}
            info = LANGUAGES[lang]
            simplename = data['simplename'] = os.path.split(dirname)[1] # dirname
            name = joinpath(dirname, simplename)
            data['name'] = name
            fin = info['fin'].format(**data) if 'fin' in info else ''
            data['fin'] = fin
            fout = info['fout'].format(**data)
            data['fout'] = fout
            if 'compiler' in info:
                program = info['compiler'].format(**data)
                self.compile(program)
            return info['prog'].format(**data)
        else:
            self.exit("Wrong language specified")

    def test(self, dirname, binary, fin, fout):
        self.info("Testing %s ... " % fin, newline=False)
        start = datetime.now()
        prog = 'cat %s | %s' % (fin, binary)
        proc = Popen(prog, stdout=PIPE, stderr=PIPE, shell=True)
        out, err = proc.communicate()
        result = out.strip()
        timedelta = datetime.now() - start

        prog2 = 'cat %s' % fout
        proc2 = Popen(prog2, stdout=PIPE, stderr=PIPE, shell=True)
        out2, err2 = proc2.communicate()
        expected = out2.strip()

        success = result == expected and proc.returncode == 0 and proc2.returncode == 0
        if success:
            self.success_continuation('OK [%.3f]s' % get_total_seconds(timedelta))
        else:
            self.error_continuation('Fail')
            if self.namespace.verbose:
                if proc.returncode != 0:
                    msg = "    Program returned with code %d:\n%s" % (
                        proc.returncode, err)
                else:
                    msg = "    Result was:\n%s\n but expected:\n%s\n" % (
                        result, expected)
                self.error_continuation(msg)

        return {
            'success': success,
            'time':get_total_seconds(timedelta),
        }

    def get_test_cases(self, dirname):
        if getattr(self.namespace, 'case', False):
            fins = [self.namespace.case]
        else:
            testdir = joinpath(dirname, 'testdata')

            suffix = '*.in'
            if self.namespace.all:
                fins = []
                for topdir, dirnames, filenames in walk(testdir):
                    fins.extend([joinpath(topdir, filename) for filename
                                 in fnmatch.filter(filenames, suffix)])
            else:
                fins = glob(joinpath(testdir, suffix))
        for fin in fins:
            yield fin, '.'.join(fin.split('.')[:-1] + ['out'])

    def compile(self, program):
        proc = Popen([program], stdout=PIPE, stderr=PIPE, shell=True)
        out, err = proc.communicate()
        if proc.returncode != 0:
            sys.stderr.write(err)
            self.exit("Error: compilation error! Tried command: %r" % program)

    def log(self, message, newline=True):
        if not self.namespace.quiet:
            print(message, end=newline and '\n' or '')

    def info(self, message, newline=True):
        text = colored(' => %s' % message, 'blue')
        self.log(text, newline)

    def info_continuation(self, message, newline=True):
        text = colored(message, 'blue')
        self.log(text, newline)

    def success(self, message, newline=True):
        text = colored(' => %s' % message, 'green')
        self.log(text, newline)

    def success_continuation(self, message, newline=True):
        text = colored(message, 'green')
        self.log(text, newline)

    def error(self, message, newline=True):
        text = ' => %s' % message
        self.error_continuation(text, newline)

    def error_continuation(self, message, newline=True):
        text = colored(message, 'red')
        self.log(text, newline)

    def exit(self, message, code=1):
        if not self.namespace.quiet:
            sys.stderr.write(colored('ERROR: ', 'red'))
            sys.stderr.write(colored(message, 'red'))
            sys.stderr.write('\n')
        sys.exit(code)





"""
This module provides utilities or wraps existing modules/packages into
tools universal across supported Python versions.
"""
from __future__ import unicode_literals
from __future__ import print_function

import sys
try:
    import unittest2 as unittest
except ImportError:
    import unittest

if sys.version_info >= (3, 2):
    def assertItemsEqual(self, result, expected, msg=None):
        self.assertEqual(sorted(result), sorted(expected), msg=msg)
    unittest.TestCase.assertItemsEqual = assertItemsEqual


try:
    from collections import OrderedDict
except ImportError:
    from monolith.utils.ordereddict import OrderedDict

try:
    unicode = unicode
except NameError:
    basestring = unicode = str

if sys.version_info < (2, 7):
    from contextlib import nested
else:
    def nested(*context_managers):
        return tuple(context_managers)


__all__ = ['unittest', 'OrderedDict', 'nested', 'unicode']




import sys
from argparse import Namespace
from porunga.commands.test import curdir
from porunga.commands.test import TestCommand
from porunga.utils.compat import unittest
from mock import call
from mock import Mock
from mock import patch
from subprocess import PIPE
from termcolor import colored


class TestTestCommand(unittest.TestCase):

    def setUp(self):
        self.command = TestCommand()
        self.command.namespace = Namespace(quiet=True, all=False, lang='python')

    # get_binary tests

    def test_get_binary(self):
        self.assertEqual(self.command.get_binary('foobar/', 'python'),
            'python foobar/foobar.py')

    def test_get_binary_in_subdirectory(self):
        self.assertEqual(self.command.get_binary('foobar/foo', 'python'),
            'python foobar/foo/foo.py')

    def test_get_binary_python(self):
        self.assertEqual(self.command.get_binary('foobar', 'python'),
            'python foobar/foobar.py')

    def test_get_binary_ruby(self):
        self.assertEqual(self.command.get_binary('foobar', 'ruby'),
            'ruby foobar/foobar.rb')

    def test_get_binary_java(self):
        self.command.compile = Mock()
        self.assertEqual(self.command.get_binary('foobar', 'java'),
            'java -cp foobar foobar')
        self.command.compile.assert_called_once_with('javac foobar/foobar.java')

    def test_get_binary_objc(self):
        self.command.compile = Mock()
        self.assertEqual(self.command.get_binary('foobar/', 'objc'),
            'foobar/foobar.m.out')
        self.command.compile.assert_called_once_with(
            'clang -ObjC -framework Foundation foobar/foobar.m -o foobar/foobar.m.out')

    def test_get_binary_c(self):
        self.command.compile = Mock()
        self.assertEqual(self.command.get_binary('foobar/', 'c'),
            'foobar/foobar.c.out')
        self.command.compile.assert_called_once_with(
            'gcc -O2 foobar/foobar.c -o foobar/foobar.c.out')

    def test_get_binary_cpp(self):
        self.command.compile = Mock()
        self.assertEqual(self.command.get_binary('foobar/', 'cpp'),
            'foobar/foobar.cpp.out')
        self.command.compile.assert_called_once_with(
            'g++ -O2 foobar/foobar.cpp -o foobar/foobar.cpp.out')

    # compile tests

    def test_compile_went_wrong(self):

        with patch('porunga.commands.test.Popen') as PopenMock:
            PopenMock.return_value = mock = Mock()
            mock.communicate = lambda: ('out', 'err')
            mock.returncode = 102
            with self.assertRaises(SystemExit) as err:
                self.command.compile('some prog')
                self.assertEqual(err.code, 102)

    def test_wrong_lang(self):
        with self.assertRaises(SystemExit):
            self.command.get_binary('foobar', 'foolang')

    def test_exit(self):
        with self.assertRaises(SystemExit) as err:
            self.command.exit('error message', 101)
            self.assertEqual(err.code, 101)

            self.command.namespace.quiet = False
            with patch.object(sys, 'stderr') as errmock:
                self.command.exit('message')
                self.assertEqual(errmock.write.call_args_list, [
                    call(colored('Error: ', 'red')),
                    call(colored('message', 'red')),
                    call('\n'),
                ])

    def test_get_test_cases(self):
        with patch('porunga.commands.test.glob') as mglob:
            mglob.return_value = ['foobar/test01.in', 'foobar/test02.in']
            self.assertItemsEqual(self.command.get_test_cases('foobar'), [
                ('foobar/test01.in', 'foobar/test01.out'),
                ('foobar/test02.in', 'foobar/test02.out'),
            ])
            self.assertEqual(mglob.call_args, call('foobar/testdata/*.in'))

    def test_get_test_cases_all_flag(self):
        with patch('porunga.commands.test.walk') as walkmock:
            walkmock.return_value = [
                ('foobar/testdata', ['bigtests'], ['test01.in', 'test01.out', 'test02.in']),
                ('foobar/testdata/bigtests', [], ['test-big-01.in', 'test02.out']),
            ]
            self.command.namespace.all = True
            test_cases = list(self.command.get_test_cases('foobar/'))

            walkmock.assert_called_once_with('foobar/testdata')
            self.assertEqual([fin for fin, fout in test_cases], [
                'foobar/testdata/test01.in',
                'foobar/testdata/test02.in',
                'foobar/testdata/bigtests/test-big-01.in',
            ])

    def test_get_test_cases_case_option(self):
        with patch('porunga.commands.test.glob') as mglob:
            mglob.return_value = ['foobar/test01.in', 'foobar/test02.in']
            self.command.namespace.case = '/foo/bar/baz/some.in'
            self.assertItemsEqual(self.command.get_test_cases('foobar'), [
                ('/foo/bar/baz/some.in',  '/foo/bar/baz/some.out')])
            self.assertFalse(mglob.called)

    def test_handle_label(self):
        with patch('porunga.commands.test.Popen') as PopenMock:
            self.command.get_test_cases = Mock(return_value=[
                ('test1.in', 'test1.out'), ('test2.in', 'test2.out')])
            mock = PopenMock.return_value = Mock()
            mock.communicate = lambda: ('out', 'err')
            mock.returncode = 0

            self.command.get_binary = Mock()
            self.command.info = Mock()
            self.command.success = Mock()

            self.command.handle_label('foobar', self.command.namespace)

            self.command.get_binary.assert_called_once_with('foobar', 'python')
            self.command.get_test_cases.assert_called_once_with('foobar')
            self.assertTrue(self.command.info.called)
            self.assertTrue(self.command.success.called)

            # check if no label was given
            with patch('porunga.commands.test.curdir') as curdirmock:
                curdirmock.return_value = '/foo/bar'
                self.command.handle_label(None, self.command.namespace)
                self.assertEqual(self.command.get_binary.call_args,
                    call('/foo/bar', 'python'))

    def test_test_program(self):
        with patch('porunga.commands.test.Popen') as PopenMock:
            mock = PopenMock.return_value = Mock()
            mock.communicate = lambda: ('out', 'err')
            mock.returncode = 0
            info = self.command.test('foobar', 'foobar/foobar.out', 'fin', 'fout')

            self.assertEqual(PopenMock.call_args_list, [
                call('cat fin | foobar/foobar.out', stdout=PIPE, stderr=PIPE,
                     shell=True),
                call('cat fout', stdout=PIPE, stderr=PIPE, shell=True),
            ])

            self.assertTrue(info['success'])
            self.assertIn('time', info)

    def test_test_program_fails(self):
        with patch('porunga.commands.test.Popen') as PopenMock:
            self.command.log = Mock()
            self.command.namespace.verbose = True
            mock = PopenMock.return_value = Mock()
            mock.communicate = lambda: ('out', 'err')
            mock.returncode = 1
            info = self.command.test('foobar', 'foobar/foobar.out', 'fin', 'fout')

            self.assertEqual(PopenMock.call_args_list, [
                call('cat fin | foobar/foobar.out', stdout=PIPE, stderr=PIPE,
                     shell=True),
                call('cat fout', stdout=PIPE, stderr=PIPE, shell=True),
            ])

            self.assertFalse(info['success'])
            self.assertIn('time', info)

    def test_logs(self):
        self.command.log = Mock()

        self.command.info('foo', False)
        self.assertEqual(self.command.log.call_args,
            call(colored(' => foo', 'blue'), False))

        self.command.info_continuation('foo', False)
        self.assertEqual(self.command.log.call_args,
            call(colored('foo', 'blue'), False))

        self.command.success('foo', False)
        self.assertEqual(self.command.log.call_args,
            call(colored(' => foo', 'green'), False))

        self.command.success_continuation('foo', False)
        self.assertEqual(self.command.log.call_args,
            call(colored('foo', 'green'), False))

        self.command.error('foo', False)
        self.assertEqual(self.command.log.call_args,
            call(colored(' => foo', 'red'), False))

        self.command.error_continuation('foo', False)
        self.assertEqual(self.command.log.call_args,
            call(colored('foo', 'red'), False))

    def test_curdir(self):
        with patch('porunga.commands.test.os') as osmock:
            osmock.path.curdir = '/foo/bar/baz'
            self.assertEqual(curdir(), '/foo/bar/baz')




import os
import sys
from setuptools import setup, find_packages

readme_file = os.path.abspath(os.path.join(os.path.dirname(__file__),
    'README.rst'))

try:
    long_description = open(readme_file).read()
except IOError as err:
    sys.stderr.write("[ERROR] Cannot find file specified as "
        "long_description (%s)\n" % readme_file)
    sys.exit(1)

extra_kwargs = {'tests_require': ['mock']}
if sys.version_info < (2, 7):
    extra_kwargs['tests_require'].append('unittest2')
if sys.version_info >= (3,):
    extra_kwargs['use_2to3'] = True

porunga = __import__('porunga')

setup(
    name='porunga',
    version=porunga.get_version(),
    url='https://github.com/lukaszb/porunga',
    author='Lukasz Balcerzak',
    author_email='lukaszbalcerzak@gmail.com',
    description=porunga.__doc__,
    long_description=long_description,
    zip_safe=False,
    packages=find_packages(),
    scripts=[],
    entry_points={
        'console_scripts': [
            'porunga = porunga:main',
        ],
    },
    test_suite='porunga.tests.collector',
    install_requires=['monolith', 'termcolor'],
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
    **extra_kwargs
)

